/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009-2010 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.actions;

import java.net.URL;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IApCoCoAConstants;
import org.apcocoa.eclipse.console.ApCoCoAConsole;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.osgi.framework.Bundle;

/**
 * This action toggles the console width, see {@link ApCoCoAConsole}. It is
 * called when the corresponding property in the preference store changes, or when the
 * user clicks on the corresponding button in the console view. Note that both
 * ways are related: When you click on the button, the property in the
 * preference store must change and vice versa.
 * 
 * As of 2010-10-06 there is an Eclipse problem with limited console width
 * enabled, see https://bugs.eclipse.org/bugs/show_bug.cgi?id=243877. We
 * therefore warn the user if she wants to enable this feature and ask for
 * explicit confirmation.
 * 
 * @author Stefan Schuster
 * 
 */
public class ToggleUseConsoleWidthAction extends Action implements
		IPropertyChangeListener {

	// private ApCoCoAConsole mConsole;

	/**
	 * 
	 * @param console
	 *            Currently not used, the parameter is probably a design error.
	 */
	public ToggleUseConsoleWidthAction(ApCoCoAConsole console) {
		super("Set limiting the CoCoA output on or off.", IAction.AS_CHECK_BOX);
		// mConsole = console;
		
		// TODO: Handle the ImageDescriptor in a centralized and resource
		// saving way, see org.eclipse.debug.internal.ui.DebugPluginImages
		ImageDescriptor desc = ImageDescriptor.getMissingImageDescriptor();
		Bundle bundle = Platform.getBundle(ApCoCoAPlugin.getPluginId());
		URL url = null;
		if (bundle != null) {
			url = FileLocator.find(bundle, new Path(
					"$nl$/images/LimitWidth.gif"), null);
			if (url != null) {
				desc = ImageDescriptor.createFromURL(url);
			}
		}
		setImageDescriptor(desc);
		setToolTipText("Limit output width.");
		getPreferenceStore().addPropertyChangeListener(this);
		update();
	}

	/**
	 * Update the state of this check box according to the state of the
	 * corresponding preference.
	 */
	private void update() {
		IPreferenceStore prefs = getPreferenceStore();
		boolean value = prefs.getBoolean(IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH);
		setChecked(value);
	}

	/**
	 * Shortcut for access to the preference store.
	 * 
	 * @return ApCoCoAPlugin's preference store.
	 */
	private IPreferenceStore getPreferenceStore() {
		return ApCoCoAPlugin.getPlugin().getPreferenceStore();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		// The state of the icon is changed by the super class,
		// here we only have to take care of the respective property,
		// we do not have to call setChecked().
		// Since we are also a IPropertyChangeListener, we play it safe and
		// unregister ourself, change the property and then re-register.
		IPreferenceStore prefs = getPreferenceStore();
		boolean state = isChecked();
		
		if (state && ! confirmLimitWidth())
			setChecked(false);
		
		state = isChecked();
		
		prefs.removePropertyChangeListener(this);
		prefs.setValue(IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH, state);
		prefs.addPropertyChangeListener(this);
	}

	/**
	 * Remove any reference to this class we know of.
	 */
	public void dispose() {
		getPreferenceStore().removePropertyChangeListener(this);
		// mConsole = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse
	 * .jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		String property = event.getProperty();

		if (property.equals(IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH)) {
			IPreferenceStore prefs = getPreferenceStore();
			boolean value = prefs.getBoolean(IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH);
			if (value && !confirmLimitWidth()) {
				prefs.removePropertyChangeListener(this);
				prefs.setValue(IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH, false);
				prefs.addPropertyChangeListener(this);
			}
			update();
		}
	}
	
	private boolean confirmLimitWidth() {
		Shell shell = Display.getDefault().getActiveShell();
		return MessageDialog.openQuestion(shell, "Warning!", 
				"Limiting console widht is currently an experimental feature and can cause ApCoCoA to crash.\n" +
				"Are you sure that you want to enable this feature?");
	}
}
