/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.cocoawrapper;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.IPatternMatchListener;
import org.eclipse.ui.console.PatternMatchEvent;
import org.eclipse.ui.console.TextConsole;
import org.eclipse.ui.progress.IProgressService;

/**
 * TODO: Thread safeness
 * 
 * @author  schuster
 */
public class CoCoAProcessManager implements IPatternMatchListener
{
	
	private Map<String, String> fFileNameMapping = new HashMap<String, String>();

	/**
	 * @uml.property  name="fCurrentProcess"
	 * @uml.associationEnd  
	 */
	private CoCoAProcess fCurrentProcess = null;
	
	/**
	 * This string is printed ("PrintLn"-ed) after every commands that
	 * are sent to CoCoA. It is used to recognize the end of a computation.
	 * Note: is used as pattern in a RegExpMatcher, beware which characters you use.
	 */
	public final String fEndOfOutputMarker = "-- Done.";
	
	private ArrayList<IComputationListener> computationListeners 
				= new ArrayList<IComputationListener>();
	
	/**
	 * @uml.property  name="fCProcessManager"
	 * @uml.associationEnd  
	 */
	private static CoCoAProcessManager fCProcessManager = null;
	
	private TextConsole console;

	private boolean isComputationRunning = false;

	private String fLastTmpFileName = "";

	private String fLastDisplayName = "";
	
	private CoCoAProcessManager() {
		
	}
	
	public static CoCoAProcessManager getCoCoAProcessManager() {
		if (fCProcessManager == null)
			fCProcessManager = new CoCoAProcessManager();
		return fCProcessManager;
	}
	
	public void startApCoCoA() {
		fCurrentProcess = CoCoAProcess.getCoCoAProcess();
		isComputationRunning = false;
	}
	
	public void stopApCoCoA() {
		if (fCurrentProcess != null) {
			if (isComputationRunning)
				fireComputationInterrupted();
			CoCoAProcess.stopCoCoA2();
			isComputationRunning = false;
		}
	}
	
	public void restartApCoCoA() {
		stopApCoCoA();
		startApCoCoA();
	}
	
	public void interruptApCoCoA() {
		if (fCurrentProcess != null) {
			fCurrentProcess.interrupt();
			if (isComputationRunning)
				fireComputationInterrupted();
			isComputationRunning = false;
		}
	}
	
	private void sendToCurrentProcess(String s) 
	throws CoCoAProcessException {
		ApCoCoAPlugin.debugLog("CoCoAProcessManager.sendToCurrentProcess(): Called.");
		try {
			fCurrentProcess.sendString(s);
			ApCoCoAPlugin.debugLog("CoCoAProcessManager.sendToCurrentProcess(): isComputationRunning = true.");
			isComputationRunning  = true;
		} catch (IOException e) {
			throw new CoCoAProcessException(e);
		}
		ApCoCoAPlugin.debugLog("CoCoAProcessManager.sendToCurrentProcess(): Leaving.");
	}
	
	/**
	 * 
	 * @param program The CoCoAL code to be sent.
	 * @param displayName A name that is associated to the temporary file that is created.
	 * 			May be <code>null</code>.
	 * @throws CoCoAProcessException
	 */
	public void sendToCoCoA(String program, String displayName) throws CoCoAProcessException {
		try {
			File tmp = File.createTempFile("GUItoCoCoA", ".coc");
			tmp.deleteOnExit();
			FileWriter fw = new FileWriter(tmp);
			BufferedWriter bw = new BufferedWriter(fw);
			bw.write(program);
			bw.close();
			ApCoCoAPlugin.getPlugin().getConsole().appendCommands(program+"\n\n");
			synchronized (fLastTmpFileName) {
				fLastTmpFileName = tmp.getAbsolutePath();
				fLastDisplayName = displayName;
			}
			fFileNameMapping.put(fLastTmpFileName, displayName);
			
			fireComputationStarted();
			sendFileToCurrentProcess(fLastTmpFileName);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		ApCoCoAPlugin.debugLog("CoCoAProcessManager.sendCmdToCurrentProcessPerTmpFile(): Leaving.");
	}
	
	private void sendFileToCurrentProcess(String rawAbsoluteFilePath) throws CoCoAProcessException {
		String source = "<< \"" + rawAbsoluteFilePath.replaceAll("\\\\", "\\\\\\\\") + "\"; Print \"" + fEndOfOutputMarker + "\";";
		sendToCurrentProcess(source);
	}
	
	public void addComputationListener(IComputationListener listener) {
		computationListeners.add(listener);
	}
	
	public void removeComputationListener(IComputationListener listener) {
		computationListeners.remove(listener);
	}
	
	private void fireComputationStarted() {
		// ApCoCoAPlugin.debugLog("CoCoAProcessManager.fireComputationStarted(): Called");
//		if (computationListeners == null)
//			ApCoCoAPlugin.debugLog("Listeners is null");
		for (IComputationListener listener : computationListeners) {
			// System.out.println(".." + listener.toString());
			listener.computationStarted();
		}
		// ApCoCoAPlugin.debugLog("CoCoAProcessManager.fireComputationStarted(): Leaving");
	}
	
	private void fireComputationFinished() {
		for (IComputationListener listener : computationListeners) {
			listener.computationFinished();
		}
	}
	
	private void fireComputationInterrupted() {
		for (IComputationListener listener : computationListeners) {
			listener.computationInterrupted();
		}
	}

	public int getCompilerFlags() {
		// TODO Auto-generated method stub
		return 0;
	}

	public String getLineQualifier() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getPattern() {
		return this.fEndOfOutputMarker;
	}

	public void connect(TextConsole console) {
		this.console = console;
	}

	public void disconnect() {
		this.console = null;
	}

	public void matchFound(PatternMatchEvent event) {
		// The thread that runs the ConsolePatternMatcher calls matchFound,
		// which in turn runs in that other thread. But the firing will
		// finally result in updating the editor status bar. Therefore,
		// we have to run that in the UI thread.
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				CoCoAProcessManager.getCoCoAProcessManager().fireComputationFinished();
			}
		});
	}
	
	public String getLastDisplayName() {
		synchronized (fLastTmpFileName) {
			return fLastDisplayName;
		}
	}
	
	public String getLastTmpFileName() {
		synchronized (fLastTmpFileName) {
			return fLastTmpFileName;
		}
	}
}
