/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.console;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IApCoCoAConstants;
import org.apcocoa.eclipse.cocoawrapper.CoCoAProcessManager;
import org.apcocoa.eclipse.console.ConsoleDocument.ConsoleLine;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.model.IFlushableStreamMonitor;
import org.eclipse.debug.core.model.IStreamMonitor;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;
import org.eclipse.ui.themes.ITheme;
import org.eclipse.ui.themes.IThemeManager;

/**
 * Code copied from CVS-Console (org.eclipse.team.internal.ccvs.ui.console.CVSOutputConsole) 
 * and adapted/extended to our needs.
 * 
 * 
 * @author Stefan Schuster
 */
public class ApCoCoAConsole extends MessageConsole implements
		IPropertyChangeListener, /* IStreamListener, */ ICoCoAConsole {

	private boolean showOnMessage;

	private ConsoleDocument document;
	private IConsoleManager consoleManager;

	// created colors for each line type - must be disposed at shutdown
	private Color commandColor;
	private Color messageColor;
	private Color errorColor;

	// streams for each command type - each stream has its own color
	private MessageConsoleStream commandStream;
	private MessageConsoleStream messageStream;
	private MessageConsoleStream errorStream;

	// Indicates whether the console is visible in the Console view
	private boolean visible = false;

	// Indicates whether the console's streams have been initialized
	private boolean initialized = false;

	public ApCoCoAConsole(String title) {
		super(title, ApCoCoAPlugin.getImageDescriptor(IApCoCoAConstants.IMG_AC));
		
		// TODO showOnMessage =
		// ApcocoaPlugin.getPlugin().getPreferenceStore().getBoolean(ICVSUIConstants.PREF_CONSOLE_SHOW_ON_MESSAGE);
		
		showOnMessage = true;
		document = new ConsoleDocument();
		consoleManager = ConsolePlugin.getDefault().getConsoleManager();
		
		// TODO
		// CVSProviderPlugin.getPlugin().setConsoleListener(CVSOutputConsole.this);
		
		updateWidth();
		
		ApCoCoAPlugin.getPlugin().getPreferenceStore()
				.addPropertyChangeListener(ApCoCoAConsole.this);
		
		// PlatformUI.getWorkbench().getThemeManager().addPropertyChangeListener(ApCoCoAConsole.this);
		
		
		addPatternMatchListener(CoCoAProcessManager.getCoCoAProcessManager());
	}

	/*
	 * Called when console is added to the console view.
	 * 
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.console.AbstractConsole#init()
	 */
	protected void init() {
		//
		super.init();

		// initLimitOutput();
		// initWrapSetting();

		updateFont();

		// Ensure that initialization occurs in the ui thread
		ApCoCoAPlugin.getStandardDisplay().asyncExec(new Runnable() {
			public void run() {
				JFaceResources.getFontRegistry().addListener(
						ApCoCoAConsole.this);
				initializeStreams();
				dump();
			}
		});
	}

	/**
	 * Initialize the streams of the console. Must be called from the UI thread.
	 */
	private void initializeStreams() {
		ApCoCoAPlugin.debugLog("Initialize streams...");
		synchronized (document) {
			if (!initialized) {
				commandStream = newMessageStream();
				errorStream = newMessageStream();
				messageStream = newMessageStream();
				// install colors
				commandColor = createColor(ApCoCoAPlugin.getStandardDisplay(),
						IApCoCoAConstants.PREF_COMMANDS_COLOR);
				commandStream.setColor(commandColor);

				messageColor = createColor(ApCoCoAPlugin.getStandardDisplay(),
						IApCoCoAConstants.PREF_RESPONSE_COLOR);
				messageStream.setColor(messageColor);

				errorColor = new Color(ApCoCoAPlugin.getStandardDisplay(),
						new RGB(200, 0, 0));
				errorStream.setColor(errorColor);
				// install font
				// Font f =
				// PlatformUI.getWorkbench().getThemeManager().getCurrentTheme().getFontRegistry().get(ICVSUIConstants.PREF_CONSOLE_FONT);
				// setFont(f);
				// cocoaProcess = CoCoAProcess.getCoCoAProcess();
				initialized = true;
			}
		}
	}

	private void dump() {
		synchronized (document) {
			visible = true;
			ConsoleDocument.ConsoleLine[] lines = document.getLines();
			for (int i = 0; i < lines.length; i++) {
				ConsoleDocument.ConsoleLine line = lines[i];
				appendLine(line.type, line.line);
			}
			document.clear();
		}
	}

	public synchronized void appendCommands(String commands) {
		appendLine(ConsoleDocument.COMMAND, commands);
	}
	
	public synchronized void appendError(String error) {
		appendLine(ConsoleDocument.ERROR, error);
	}

	protected void appendLine(int type, String line) {
		showConsole();
		System.out.println("ApCoCoAConsole.appendLine(): " + line);
		synchronized (document) {
			if (visible) {
				switch (type) {
				case ConsoleDocument.COMMAND:
					commandStream.print(line);
					break;
				case ConsoleDocument.MESSAGE:
					messageStream.print(line); //$NON-NLS-1$
					break;
				case ConsoleDocument.ERROR:
					errorStream.print(line); //$NON-NLS-1$
					break;
				}
			} else {
				document.appendConsoleLine(type, line);
			}
		}
	}

	private void showConsole() {
		show(false);
	}

	/**
	 * Show the console.
	 * 
	 * @param showNoMatterWhat
	 *            ignore preferences if <code>true</code>
	 */
	public void show(boolean showNoMatterWhat) {
		if (showNoMatterWhat || showOnMessage) {
			if (visible)
				consoleManager.showConsoleView(this);
			else
				ApCoCoAConsoleFactory.showConsole(this);
		}
	}

	/**
	 * Returns a color instance based on data from a preference field.
	 */
	private Color createColor(Display display, String preference) {
		RGB rgb = PreferenceConverter.getColor(ApCoCoAPlugin.getPlugin()
				.getPreferenceStore(), preference);
		return new Color(display, rgb);
	}

	public void propertyChange(PropertyChangeEvent event) {
		String property = event.getProperty();
		if (property.equals(IApCoCoAConstants.PREF_COMMANDS_COLOR)) {
			Color newColor = createColor(ApCoCoAPlugin.getStandardDisplay(),
					IApCoCoAConstants.PREF_COMMANDS_COLOR);
			commandStream.setColor(newColor);
			commandColor.dispose();
			commandColor = newColor;
		} else if (property.equals(IApCoCoAConstants.PREF_RESPONSE_COLOR)) {
			Color newColor = createColor(ApCoCoAPlugin.getStandardDisplay(),
					IApCoCoAConstants.PREF_RESPONSE_COLOR);
			messageStream.setColor(newColor);
			messageColor.dispose();
			messageColor = newColor;
		} else if (property
				.equals("org.apcocoa.eclipse.fontDefinition.ApCoCoAconsole")) {
			updateFont();
		} else if (property.equals(IApCoCoAConstants.PREF_CONSOLE_WIDTH)
				|| property.equals(IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH)) {
			updateWidth();
		}
	}

	private void updateWidth() {
		boolean isUsingWidth = ApCoCoAPlugin.getPlugin().getPreferenceStore()
				.getBoolean(IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH);
		int newWidth = ApCoCoAPlugin.getPlugin().getPreferenceStore()
				.getInt(IApCoCoAConstants.PREF_CONSOLE_WIDTH);
		if (isUsingWidth)
			this.setConsoleWidth(newWidth);
		else
			this.setConsoleWidth(0);
	}
	
	private void updateFont() {
		IThemeManager themeManager = PlatformUI.getWorkbench()
				.getThemeManager();
		ITheme currentTheme = themeManager.getCurrentTheme();
		FontRegistry fontReg = currentTheme.getFontRegistry();
		Font f = fontReg
				.get("org.apcocoa.eclipse.fontDefinition.ApCoCoAconsole");
		this.setFont(f);
	}

	/**
	 * Used to notify this console of lifecycle methods <code>init()</code> and
	 * <code>dispose()</code>.
	 */
	public class MyLifecycle implements org.eclipse.ui.console.IConsoleListener {
		public void consolesAdded(IConsole[] consoles) {
			for (int i = 0; i < consoles.length; i++) {
				IConsole console = consoles[i];
				if (console == ApCoCoAConsole.this) {
					init();
				}
			}

		}

		public void consolesRemoved(IConsole[] consoles) {
			for (int i = 0; i < consoles.length; i++) {
				IConsole console = consoles[i];
				if (console == ApCoCoAConsole.this) {
					ConsolePlugin.getDefault().getConsoleManager()
							.removeConsoleListener(this);
					dispose();
				}
			}
		}
	}

//	public synchronized void streamAppended(String text, IStreamMonitor monitor) {
//		synchronized (monitor) {
//			appendLine(ConsoleDocument.MESSAGE, text);
//			if (monitor instanceof IFlushableStreamMonitor)
//				((IFlushableStreamMonitor) monitor).flushContents();
//		}
//	}

	/**
	 * Clean-up created fonts.
	 */
	public void shutdown() {
		// Call super dispose because we want the partitioner to be
		// disconnected.
		super.dispose();
		if (commandColor != null)
			commandColor.dispose();
		if (messageColor != null)
			messageColor.dispose();
		if (errorColor != null)
			errorColor.dispose();
		ApCoCoAPlugin.getPlugin().getPreferenceStore()
				.removePropertyChangeListener(this);
	}

	public String getContent() {
		StringBuffer sb = new StringBuffer();
		ConsoleLine[] cLines = document.getLines();
		for (int i = 0; i < cLines.length; i++) {
			sb.append(cLines[i].line);
		}
		return sb.toString();
	}


	public synchronized void appendResponse(String response) {
		appendLine(ConsoleDocument.MESSAGE, response);
		
	}
}
