/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.editors;

import org.apcocoa.eclipse.core.ACPackageManager;
import org.apcocoa.eclipse.templates.ApCoCoAContextType;
import org.eclipse.jface.internal.text.html.HTMLPrinter;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.DefaultInformationControl;
import org.eclipse.jface.text.IAutoEditStrategy;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IInformationControl;
import org.eclipse.jface.text.IInformationControlCreator;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextHover;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.Region;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.ITokenScanner;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.jface.text.templates.GlobalTemplateVariables;
import org.eclipse.jface.text.templates.Template;
import org.eclipse.jface.text.templates.TemplateCompletionProcessor;
import org.eclipse.jface.text.templates.TemplateContext;
import org.eclipse.jface.text.templates.TemplateContextType;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.editors.text.TextSourceViewerConfiguration;


/**
 * @author  Stefan Schuster
 */
public class ApCoCoAConfiguration extends TextSourceViewerConfiguration {
	// private XMLDoubleClickStrategy doubleClickStrategy;
	// private XMLTagScanner tagScanner;
	private RuleBasedScanner codeScanner = null;
	private RuleBasedScanner commentScanner = null;
	private RuleBasedScanner stringScanner = null;
	/**
	 * @uml.property  name="colorManager"
	 * @uml.associationEnd  
	 */
	private ColorManager colorManager;
	
	private final String[] keywords = {
			"for", 
			"define",
			"enddefine",
			"endfor", 
			"while", 
			"endwhile", 
			"do", 
			"if", 
			"endif",
			"then",
			"else",
			"elsif",
			"alias",
			"break",
			"foreach",
			"endforeach",
			"repeat",
			"endrepeat",
			"print",
			"println",
			"to",
			"or",
			"not"};
	
	public ApCoCoAConfiguration(ColorManager colorManager) {
		this.colorManager = colorManager;
	}

	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] {
			IDocument.DEFAULT_CONTENT_TYPE,
			ApCoCoAPartitionScanner.APCOCOA_COMMENT };
	}
	
	/**
	 * @return
	 * @uml.property  name="codeScanner"
	 */
	private ITokenScanner getCodeScanner() {
		if (codeScanner == null) {
			IToken defaultToken = 
				new Token(
						new TextAttribute(colorManager.getColor(ApCoCoAColorConstants.DEFAULT)));
//			IToken stringToken =
//				new Token(
//						new TextAttribute(colorManager.getColor(ApCoCoAColorConstants.STRING)));
			IToken keywordToken =
				new Token(
						new TextAttribute(colorManager.getColor(ApCoCoAColorConstants.KEYWORD), null, SWT.BOLD));
			
			WordRule wrule = new WordRule( new IWordDetector() {
					public boolean isWordPart(char c) {
						return c == '-' || Character.isLetter(c) || Character.isDigit(c);
					}
					public boolean isWordStart(char c) {
						return c == '-' || Character.isLetter(c);
					}
				},
				// Token.UNDEFINED,
				defaultToken,
				true);
			for (int i=0; i<keywords.length; i++) {
				wrule.addWord(keywords[i], keywordToken);
			}
			IRule[] rules = new IRule[1];
			// rules[0] = new SingleLineRule("\"", "\"", stringToken, '\\');
			// rules[1] = new SingleLineRule("'", "'", stringToken, '\\');
			// rules[0] = new MultiLineRule("\"", "\"", stringToken, (char) 0, true);
			// rules[1] = new MultiLineRule("'", "'", stringToken, (char) 0, true);
			rules[0] = wrule;
			codeScanner = new RuleBasedScanner();
			codeScanner.setRules(rules);
			codeScanner.setDefaultReturnToken(defaultToken);
		}
		return codeScanner;
	}
	
	/**
	 * @return
	 * @uml.property  name="commentScanner"
	 */
	private ITokenScanner getCommentScanner() {
		if (commentScanner == null) {
			IToken defaultToken = 
				new Token(
						new TextAttribute(colorManager.getColor(ApCoCoAColorConstants.COMMENT)));
			commentScanner = new RuleBasedScanner();
			commentScanner.setRules(new IRule[0]);
			commentScanner.setDefaultReturnToken(defaultToken);
		}
		return commentScanner;
	}
	
	/**
	 * @return
	 * @uml.property  name="commentScanner"
	 */
	private ITokenScanner getStringScanner() {
		if (stringScanner == null) {
			IToken defaultToken = 
				new Token(
						new TextAttribute(colorManager.getColor(ApCoCoAColorConstants.STRING)));
			stringScanner = new RuleBasedScanner();
			stringScanner.setRules(new IRule[0]);
			stringScanner.setDefaultReturnToken(defaultToken);
		}
		return stringScanner;
	}
	
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {
		PresentationReconciler reconciler = new PresentationReconciler();
		
		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(getCodeScanner());
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		
		dr = new DefaultDamagerRepairer(getCommentScanner());
		reconciler.setRepairer(dr, ApCoCoAPartitionScanner.APCOCOA_COMMENT);
		reconciler.setDamager(dr, ApCoCoAPartitionScanner.APCOCOA_COMMENT);
		
		dr = new DefaultDamagerRepairer(getStringScanner());
		reconciler.setRepairer(dr, ApCoCoAPartitionScanner.APCOCOA_STRING);
		reconciler.setDamager(dr, ApCoCoAPartitionScanner.APCOCOA_STRING);

		return reconciler;
	}
	
	public ITextHover getTextHover(ISourceViewer sourceViewer, String contentType) {
		// TODO: Extend functionality, create in a separate class
		ITextHover textHover = new ITextHover() {
			public IRegion getHoverRegion(ITextViewer textViewer, int offset) {
				IDocument doc = textViewer.getDocument();
				int start = offset;
				try {
					while (start > 0) {
						if (Character.isWhitespace(doc.getChar(start))) break;
						--start;
					}
				} catch (BadLocationException e) {
				}
				int end = offset;
				try {
					while (end < doc.getLength()) {
						if (Character.isWhitespace(doc.getChar(end))) break;
						++end;
					}
				} catch (BadLocationException e) {
				}
				return new Region(start, end - start);
			}
			
			public String getHoverInfo(ITextViewer textViewer, IRegion hoverRegion) {
				try {
					String s =  
						textViewer.getDocument()
						.get(hoverRegion.getOffset(), hoverRegion.getLength());
					return s;
				} catch (BadLocationException e) {
				}
				return null;
			}
		};
		return textHover;
	}
	
	public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {
		// System.out.println("getContentAssistant called.");
		ApCoCoAContentAssistant assistant = new ApCoCoAContentAssistant();
		assistant.enableAutoActivation(false);
		assistant.enableAutoInsert(false);
		assistant.setContextInformationPopupOrientation(IContentAssistant.CONTEXT_INFO_ABOVE);
		assistant.setInformationControlCreator(new IInformationControlCreator() {
			public IInformationControl createInformationControl(Shell parent) {
				return new DefaultInformationControl(parent);
			}
		});
		
		assistant.setContentAssistProcessor(new ApCoCoAConentAssistProcessor(assistant), IDocument.DEFAULT_CONTENT_TYPE);
		
		// IContentAssistProcessor processor = getContentAssistProcessor();
		// TemplateContext
		// assistant.setContentAssistProcessor(processor, IDocument.DEFAULT_CONTENT_TYPE);
		return assistant;
	}
	
	@Override
	public IAutoEditStrategy[] getAutoEditStrategies(
			ISourceViewer sourceViewer, String contentType) {
		String partitioning= getConfiguredDocumentPartitioning(sourceViewer);
		if (IDocument.DEFAULT_CONTENT_TYPE.equals(contentType))
			return new IAutoEditStrategy[] {new ApCoCoAAutoIndentStrategy(partitioning)};
		else
			return super.getAutoEditStrategies(sourceViewer, contentType);
	}
	
	
	private IContentAssistProcessor getContentAssistProcessor() {
		
		IContentAssistProcessor processor = new TemplateCompletionProcessor() {
			protected Template[] getTemplates(String contextTypeId) {
				return ACPackageManager.getFunctionCompletionTemplates();
			}
			protected TemplateContextType getContextType(ITextViewer viewer, IRegion region) {
				// TemplateContextType type = new TemplateContextType("de", "Sample");
				TemplateContextType type = new ApCoCoAContextType();
				// type.addResolver(new GlobalTemplateVariables.Cursor());
				// type.addResolver(new GlobalTemplateVariables.User());
				return type;
			}
			protected Image getImage(Template template) {
				return null;
			}
		};
		return processor;
	}
}
