/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.editors;

import java.util.Calendar;
import java.util.Date;
import java.util.ResourceBundle;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.cocoawrapper.CoCoAProcessException;
import org.apcocoa.eclipse.cocoawrapper.CoCoAProcessManager;
import org.apcocoa.eclipse.cocoawrapper.IComputationListener;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IStorageEditorInput;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.contexts.IContextService;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.ContentAssistAction;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.IEditorStatusLine;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
import org.eclipse.ui.themes.ITheme;
import org.eclipse.ui.themes.IThemeManager;


/**
 * 
 * 
 * @author  Stefan Schuster
 */
public class ApCoCoAEditor extends TextEditor implements IPropertyChangeListener, IComputationListener {
	
	private StyledText inputArea;
	
	private Date startOfComputation;
	/**
	 * @uml.property  name="fCommandHistory"
	 * @uml.associationEnd  
	 */
	private CommandHistory fCommandHistory;
	
	/**
	 * @uml.property  name="colorManager"
	 * @uml.associationEnd  
	 */
	private ColorManager colorManager;
	
	/** Interactive windows have a history, get cleared after Strg+Enter etc. */
	private boolean isInteractive = true;
	
	
	private IEditorStatusLine statusLine = null;
	
	public ApCoCoAEditor() {
		super();
		// We need different providers to distinguisch between 
		// documents inside and outside the workspace.
		// setDocumentProvider(new ApCoCoADocumentProvider());
	}

	protected void initializeEditor() {
		super.initializeEditor();
		colorManager = new ColorManager();
		setSourceViewerConfiguration(new ApCoCoAConfiguration(colorManager));
		
	}
	
	@Override
	public boolean isDirty() {
		if (isInteractive)
			return false;
		else
			return super.isDirty();
	}

	@Override
	public boolean isSaveAsAllowed() {
		if (isInteractive)
			return false;
		else
			return super.isSaveAsAllowed();
	}

	@Override
	public boolean isSaveOnCloseNeeded() {
		if (isInteractive)
			return false;
		else 
			return super.isSaveOnCloseNeeded();
	}
	
	@Override
	public void createPartControl(Composite parent) {
		super.createPartControl(parent);
		// System.out.println("Create Part Control.");
		fCommandHistory = new CommandHistory();
		
		inputArea = this.getSourceViewer().getTextWidget();
		
		IThemeManager themeManager = PlatformUI.getWorkbench().getThemeManager();
		themeManager.addPropertyChangeListener(this);
		
		updateColorsAndFonts();
		
		IContextService contextService = (IContextService) getSite().getService(IContextService.class);
		// contextService.activateContext("org.eclipse.ui.textEditorScope");
		
		contextService.activateContext("org.apcocoa.eclipse.contexts.ApCoCoA");
		
		CoCoAProcessManager.getCoCoAProcessManager().addComputationListener(this);
		
		// FIXME Considered as a hack: Make an editor window interactive in dependence of its name.
		IEditorInput editorInput = this.getEditorInput();
		if (editorInput instanceof IEditorInput) {
			String name = ((IEditorInput) editorInput).getName();
			this.isInteractive = name.endsWith(".aci") || name.endsWith(".aco");
		}
	}
	
	public void forward() {
		if (!isInteractive)
			return;
		String newContent = fCommandHistory.forward();
		inputArea.setText((newContent == null) ? "" : newContent);
	}
	
	public void back() {
		if (!isInteractive)
			return;
		String newContent = fCommandHistory.back();
		inputArea.setText(newContent);
	}
	
	public void sendContent() {
		ApCoCoAPlugin.debugLog("ApCoCoAEditor.sendContent(): Called.");
		
		// Feature as in the QT-GUI: 
		// * If there is some selected part,
		//   send only this part to CoCoA.
		// * If in the interactive editor window some part is selected, 
		//   do not clear the editor window.
		String commands = inputArea.getSelectionText();
		boolean nothingSelected = false;
		if (commands.equals("")) {
			commands = inputArea.getText();
			nothingSelected = true;
		}
		try {
			String fileName = getEditorInput().getName();
			CoCoAProcessManager.getCoCoAProcessManager().sendToCoCoA(commands, fileName);
			if (isInteractive && nothingSelected) {
				fCommandHistory.enter(commands);
				inputArea.setText("");
			}
		} catch (CoCoAProcessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		ApCoCoAPlugin.debugLog("ApCoCoAEditor.sendContent(): Leaving.");
	}
	
	public void clearHistory() {
		
	}
	
	public void cancelHistoryBrowsing() {
		
	}
	
	public String getText() {
		return inputArea.getText();
	}
	
	public void dispose() {
		// System.out.println("Dispose called.");
		colorManager.dispose();
		CoCoAProcessManager.getCoCoAProcessManager().removeComputationListener(this);
		IThemeManager themeManager = PlatformUI.getWorkbench().getThemeManager();
		themeManager.removePropertyChangeListener(this);
		fCommandHistory = null;
		inputArea = null;
		statusLine = null;
		super.dispose();
	}
	
	protected void editorContextMenuAboutToShow(IMenuManager menu) {
		super.editorContextMenuAboutToShow(menu);
		addAction(menu, "ContentAssistProposal"); //$NON-NLS-1$
		// addAction(menu, "ContentAssistTip"); //$NON-NLS-1$
		// addAction(menu, "DefineFoldingRegion");  //$NON-NLS-1$
	}
	
	protected void createActions() {
		super.createActions();
		// IAction a= new TextOperationAction(ApCoCoAEditorMessages.getResourceBundle(), "ContentAssistProposal.", this, ISourceViewer.CONTENTASSIST_PROPOSALS); //$NON-NLS-1$
		// a.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS);
		// setAction("ContentAssistProposal", a); //$NON-NLS-1$
		
		String actionId = "ContentAssistProposal";
		ResourceBundle rb = Messages.getResourceBundle();
		IAction action = new ContentAssistAction(rb, "ContentAssistProposal.", this); //$NON-NLS-1$
		action.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS);
		setAction(actionId, action); //$NON-NLS-1$
		// markAsStateDependentAction(actionId, true); //$NON-NLS-1$
		// PlatformUI.getWorkbench().getHelpSystem().setHelp(action, helpContextId);
	}
	
	private void updateColorsAndFonts() {
		IThemeManager themeManager = PlatformUI.getWorkbench().getThemeManager();
		ITheme currentTheme = themeManager.getCurrentTheme();
		FontRegistry fontReg = currentTheme.getFontRegistry();
		Font f = fontReg.get("org.apcocoa.eclipse.fontDefinition.editor");
		inputArea.setFont(f);
	}

	public void propertyChange(PropertyChangeEvent event) {
		updateColorsAndFonts();
	}

	private synchronized void setStatusLine(String s) {
		// TODO: initialization into createPartControl?
		if (statusLine == null)
			statusLine = (IEditorStatusLine) this.getAdapter(IEditorStatusLine.class);
		// beware: statusLine could still not be created.
		if (statusLine != null) // BUG-FIX: 
			statusLine.setMessage(false, s, null);
	}
	
	public void computationFinished() {
		Date endOfComputation = Calendar.getInstance().getTime();
		float duration = endOfComputation.getTime() - startOfComputation.getTime();
		setStatusLine("Calculation finished (" + duration/1000 + " s)");
	}

	public void computationStarted() {
		// System.out.println("computationStarted() > " + this.getTitle());
		try {
			setStatusLine("Calculation in progress....");
			startOfComputation = Calendar.getInstance().getTime();
		} catch (NullPointerException e) {
			e.printStackTrace();
			// System.out.println(e.getMessage());
		}
		// System.out.println("computationStarted:end");
	}
	
	public void computationInterrupted() {
		setStatusLine("Computation interrupted.");
	}
	
	/**
	 * Solution for the "open workspace-external documents" found under
	 * http://stackoverflow.com/questions/455210/eclipse-editor-plugin-error-when-opening-file-outside-project
	 * 
	 * @param input
	 * @return
	 */
	private IDocumentProvider createDocumentProvider(IEditorInput input) {
		ApCoCoAPlugin.debugLog("createDocumentProvider called.");
		if (input instanceof IFileEditorInput) {
			ApCoCoAPlugin.debugLog("Got IFileEditorInput");
			return new ApCoCoADocumentProvider();
		} else if (input instanceof IStorageEditorInput) {
			ApCoCoAPlugin.debugLog("Got IStorageEditorInput");
			return new ApCoCoAStorageDocumentProvider();
		} else {
			ApCoCoAPlugin.debugLog("Got 'else': " + input.getName());
			// return new ApCoCoADocumentProvider();
			return new ApCoCoAStorageDocumentProvider();
		}
	}
	
	protected final void doSetInput(IEditorInput input) throws CoreException {
		ApCoCoAPlugin.debugLog("doSetInput " + input.getName());
		setDocumentProvider(createDocumentProvider(input));
        super.doSetInput(input);

	}
}


