/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.preferences;

import org.eclipse.debug.internal.ui.SWTFactory;
import org.eclipse.jface.preference.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.IWorkbench;
import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IApCoCoAConstants;
import org.apcocoa.eclipse.core.ACPackageManager;
import org.apcocoa.eclipse.core.RcFileManager;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 * 
 * ST: Currently, preferences are applied only at the next
 *     start.
 * 
 */

public class ApcocoaPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	private ColorFieldEditor commandColorEditor;
	private ColorFieldEditor responseColorEditor;
	private FileFieldEditor cocoaPath;
	// private FileFieldEditor cocoaServerPath;
	private StringFieldEditor cocoaServerPath;
	// private StringFieldEditor cocoaServerMachine;
	private StringFieldEditor cocoaPort;
	private DirectoryFieldEditor cocoaPackagesPath;
	private FileFieldEditor cocoaWordlistPath;
	private FileFieldEditor cocoaRcPath;
	private BooleanFieldEditor cocoaIsAutoStart;
	private IntegerFieldEditor consoleWidthEditor;
	private BooleanFieldEditor cocoaIsUsingWidth;
	
	public ApcocoaPreferencePage() {
		super(GRID);
		setPreferenceStore(ApCoCoAPlugin.getDefault().getPreferenceStore());
		setDescription("The ApCoCoA preference page");
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		
		// CoCoA executable path
		cocoaPath = new FileFieldEditor (
				IApCoCoAConstants.PREF_AC_EXEC, 
				"&CoCoA path:", 
				true,
				StringButtonFieldEditor.VALIDATE_ON_FOCUS_LOST, 
				getFieldEditorParent());
		addField(cocoaPath);
		
		// CoCoA server executable path
		Composite c = getFieldEditorParent();
		cocoaServerPath = new FileFieldEditor (
				IApCoCoAConstants.PREF_AC_SERVER_EXEC, 
				"&CoCoA server path:", 
				true,
				StringButtonFieldEditor.VALIDATE_ON_FOCUS_LOST, 
				c);
		cocoaServerPath.getTextControl(c).setSize(10, 10);
		addField(cocoaServerPath);
		
		createSpace();
		
		// CoCoA server port
		cocoaPort = new StringFieldEditor(
				IApCoCoAConstants.PREF_AC_SERVER_PORT, 
				"ApCoCoA Server Port", 
				10, 
				getFieldEditorParent());
		addField(cocoaPort);
		
		// CoCoA packages path
		cocoaPackagesPath = new DirectoryFieldEditor(
				IApCoCoAConstants.PREF_AC_PACKAGES_PATH, 
				"&CoCoA packages path", 
				getFieldEditorParent());
		addField(cocoaPackagesPath);
		
		// CoCoA wordlist path
		cocoaWordlistPath = new FileFieldEditor(
				IApCoCoAConstants.PREF_AC_WORDLIST_FILE_PATH,
				"&CoCoA wordlist path",
				true,
				StringButtonFieldEditor.VALIDATE_ON_KEY_STROKE,
				getFieldEditorParent());
		addField(cocoaWordlistPath);
		
		// path to cocoarc config file 
		cocoaRcPath = new FileFieldEditor(
				IApCoCoAConstants.PREF_AC_COCOARC_PATH, 
				"CoCoA config file", 
				true, 
				getFieldEditorParent());
		addField(cocoaRcPath);
		
		// Console stuff
		
		// Command Color
		commandColorEditor = new ColorFieldEditor(IApCoCoAConstants.PREF_COMMANDS_COLOR,
				"Command color:", getFieldEditorParent()); 
		addField(commandColorEditor);
		
		// Response Color
		responseColorEditor = new ColorFieldEditor(IApCoCoAConstants.PREF_RESPONSE_COLOR,
				"Response color:", getFieldEditorParent()); 
		addField(responseColorEditor);
		
		// Console Width
		consoleWidthEditor = new IntegerFieldEditor(
				IApCoCoAConstants.PREF_CONSOLE_WIDTH, 
				"Console width", getFieldEditorParent());
		addField(consoleWidthEditor);
		
		// Do we want to use the console width? Can be toggled via
		// Console
		cocoaIsUsingWidth = new BooleanFieldEditor(
				IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH, 
				"Turn limiting width on.", 
				getFieldEditorParent());
		addField(cocoaIsUsingWidth);
		
		// CoCoA Autostart
		cocoaIsAutoStart = new BooleanFieldEditor(
				IApCoCoAConstants.PREF_COCOA_AUTOSTART, 
				"Start (Ap)CoCoA automatically.", 
				getFieldEditorParent());
		addField(cocoaIsAutoStart);
		
		/* addField(new DirectoryFieldEditor(PreferenceConstants.P_PATH, 
				"&Directory preference:", getFieldEditorParent()));
		addField(
			new BooleanFieldEditor(
				PreferenceConstants.P_BOOLEAN,
				"&An example of a boolean preference",
				getFieldEditorParent()));

		addField(new RadioGroupFieldEditor(
				PreferenceConstants.P_CHOICE,
			"An example of a multiple-choice preference",
			1,
			new String[][] { { "&Choice 1", "choice1" }, {
				"C&hoice 2", "choice2" }
		}, getFieldEditorParent()));
		addField(
			new StringFieldEditor(PreferenceConstants.P_STRING, "A &text preference:", getFieldEditorParent()));
		*/
	}
	
//	protected Point doComputeSize() {
//		super.doComputeSize();
//		return new Point(0,0 );
//	}
	
	private void createSpace() {
		Label label= new Label(getFieldEditorParent(), SWT.NONE);
		GridData gd= new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gd.horizontalSpan= 3;
		label.setLayoutData(gd);
	}
	
	@Override
	protected void performApply() {
		super.performApply();
		ACPackageManager.initializeACPackages(true);
		RcFileManager.initRcFile();
	}

	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}
	
}